
/*
 *  agsfile.cpp
 *
 *  A class to convert bitmaps to PocketPicture .AGS files and vice versa
 *  Written by Moritz Beutel
 *  (c) 2007 by AUDACIA Software
 */

#include "agsfile.hpp"
#include <windows.h>
#include <cstring>
#include <string>
#include <stdexcept>
#include <cmath>
#include <cstddef>
#include <cstdlib>
#include <cstring>
#include <ucl/stl/arrayptr.hpp>
#include "winutil.hpp"
#include "charmap.h"



namespace apv
{


/*template <typename T>
    static void writeToFile (HANDLE hFile, const T& val)
{
    DWORD nobw;
    WriteFile (hFile, &val, sizeof (T), &nobw, NULL);
}

template <typename T>
    static T readFromFile (HANDLE hFile)
{
    DWORD nobr;
    T val;
    ReadFile (hFile, &val, sizeof (T), &nobr, NULL);
    return val;
}

static void writeToFile (HANDLE hFile, const char* string)
{
    DWORD nobw;
    WriteFile (hFile, string, std::strlen (string) + 1, &nobw, NULL);
}

static std::string readFromFile (HANDLE hFile)
{
    std::string str;
    char c;
    DWORD nobr;

    str.reserve (40);
    do
    {
        ReadFile (hFile, &c, 1, &nobr, NULL);
        str += c;
    }
    while (c != 0);

    return str;
}*/

template <typename T>
    static void writeToBlock (unsigned char*& dptr, const T& val);

template <typename T>
    static void writeToBlock (unsigned char*& dptr, const T& val)
{
    *reinterpret_cast <T*> (dptr) = val;
    dptr += sizeof (T);
}

template <typename T>
    static T readFromBlock (unsigned char*& dptr)
{
    T val = *reinterpret_cast <T*> (dptr);
    dptr += sizeof (T);
    return val;
}

template <>
    static void writeToBlock <const char*> (unsigned char*& dptr, const char* const& string)
{
    unsigned l = strlen (string) + 1;
    std::memcpy (dptr, string, l);
    dptr += l;
}

template <>
    static const char* readFromBlock <const char*> (unsigned char*& dptr)
{
    const char* r = dptr;
    dptr += strlen (r) + 1;
    return r;
}


/*template <>
    static void writeToBlock <std::string> (unsigned char*& dptr, const std::string& str)
{
    unsigned l = str.size () + 1;
    std::memcpy (dptr, str.c_str (), l);
    dptr += l;
}

template <>
    static std::string readFromBlock <std::string> (unsigned char*& dptr)
{
    std::string str = reinterpret_cast <char*> (dptr);
    dptr += str.size () + 1;

    return str;
}*/


AGSFile::AGSFile (void)
{ filesize[0] = filesize[1] = filesize[2] = 0; }
AGSFile::AGSFile (const AGSFile& rhs)
{
    for (unsigned n = 0; n < 3; ++n)
    {
        files[n].copy (rhs.files[n]);
        filesize[n] = rhs.filesize[n];
    }
}
AGSFile::~AGSFile (void)
{}


/*std::string AGSFile::getFileName (void)
{
    if (filesize[0] == 0)
        return 0;
    return files[0].get ();
}*/

void AGSFile::pasteFromBitmap (const BITMAPINFOHEADER* bmih, const BYTE* bdata,
    const char* filename)
{
    const RGBQUAD* ctbl;

    unsigned char* fptr;
    unsigned char* fbeg;
    char pvfn[220];
    short version = 101;
    short fcnt = 3;
    short xs = 160;
    short ys = 146;
    short bcnt = 0;

    unsigned slen;

    unsigned x = 0;
    const BYTE* src;
    const BYTE* xsrc;

    short ges3bcnt;
    unsigned temp1, temp2;
    unsigned srcrowlen;


    if (!(bmih && bdata))
        return;

    if ((bmih->biWidth != 160) || (bmih->biHeight != 146))
        throw std::runtime_error ("Wrong size (required: 160x146)");
    if ((bmih->biBitCount != 8) && (bmih->biClrUsed != 8))
        throw std::runtime_error ("Bitmap has wrong format");
    
    strcpy (pvfn, filename);
    for (char* p = pvfn + strlen (pvfn) - 1; p >= pvfn; --p)
        *p = static_cast <char> (charmap_ansi2pv[static_cast <unsigned char> (*p)]);
    strcat (pvfn, ".ags");
    slen = strlen (pvfn);

    _filename = filename;
    
    ctbl = reinterpret_cast <const RGBQUAD*>
        (reinterpret_cast <const char*> (bmih) + bmih->biSize);

    ges3bcnt = xs / 8 * ys;

    srcrowlen = ((bmih->biWidth * bmih->biBitCount + 31) & ~31) >> 3;
    src = bdata + srcrowlen * 145;
    xsrc = src;


    for (unsigned n = 0; n < 3; ++n)
    {
        files[n].alloc (3072);
        filesize[n] = 0;

        fbeg = fptr = files[n].get ();

        writeToBlock <const char*> (fptr, pvfn);
        if (n == 0)
        {
            writeToBlock (fptr, version);
            writeToBlock (fptr, xs);
            writeToBlock (fptr, ys);

            for (unsigned i = 0; i < 8; ++i)
                writeToBlock (fptr, ctbl[i].rgbBlue);

            writeToBlock (fptr, fcnt);
        }

        bcnt = std::min <short> ((3008 - (fptr - fbeg)) / 3, ges3bcnt);
        writeToBlock (fptr, bcnt);

        for (short b = 0; b < bcnt; ++b)
        {
            temp1 = 7 - xsrc[5];
            temp2 = 7 - xsrc[2];
            writeToBlock <unsigned char> (fptr,  (7 - xsrc[7])
                                              | ((7 - xsrc[6]) << 3)
                                              | (temp1         << 6));
            writeToBlock <unsigned char> (fptr, (temp1         >> 2)
                                              | ((7 - xsrc[4]) << 1)
                                              | ((7 - xsrc[3]) << 4)
                                              | (temp2         << 7));
            writeToBlock <unsigned char> (fptr, (temp2         >> 1)
                                              | ((7 - xsrc[1]) << 2)
                                              | ((7 - xsrc[0]) << 5));

            xsrc += 8;
            x += 8;
            if (x >= 160)
            {
                x = 0;
                src -= srcrowlen;
                xsrc = src;
            }
        }

        ges3bcnt -= bcnt;
        filesize[n] = fptr - fbeg;

        pvfn[slen]     = '~';
        pvfn[slen + 1] = '1' + n;
        pvfn[slen + 2] = '\0';
    }
}

void AGSFile::getAsBitmap (ucl::stl::ArrayPtr <BITMAPINFOHEADER>& bmih,
                  ucl::stl::ArrayPtr <BYTE>& bdata)
{
    unsigned dstrowlen;
    RGBQUAD* ctbl;
    unsigned x = 0;
    unsigned char* dst;
    unsigned char* xdst;
    unsigned char* fptr;
    short bcnt;

    unsigned char bytes[3];


    if (filesize[0] == 0)
        return;

    bmih.alloc (sizeof (BITMAPINFOHEADER) + sizeof (RGBQUAD) * 8);

    ctbl = reinterpret_cast <RGBQUAD*>
        (reinterpret_cast <char*> (bmih.get ()) + sizeof (BITMAPINFOHEADER));

    dstrowlen = ((160 * 8 + 31) & ~31) >> 3;

    bmih->biSize = sizeof (BITMAPINFOHEADER);
    bmih->biWidth = 160;
    bmih->biHeight = 146;
    bmih->biPlanes = 1;
    bmih->biBitCount = 8;
    bmih->biCompression = BI_RGB;
    bmih->biSizeImage = dstrowlen * 146;
    bmih->biXPelsPerMeter = bmih->biYPelsPerMeter = 0;
    bmih->biClrUsed = 8;
    bmih->biClrImportant = 8;

    bdata.alloc (bmih->biSizeImage);

    xdst = dst = bdata.get () + dstrowlen * 145; 

    for (unsigned n = 0; n < 3; ++n)
    {
        fptr = files[n].get ();

        readFromBlock <const char*> (fptr);

        if (n == 0)
        {
            readFromBlock <short> (fptr); // version
            readFromBlock <short> (fptr); // xs
            readFromBlock <short> (fptr); // ys

            for (unsigned i = 0; i < 8; ++i)
            {
                ctbl[i].rgbBlue = ctbl[i].rgbGreen = ctbl[i].rgbRed
                    = readFromBlock <unsigned char> (fptr);
                ctbl[i].rgbReserved = 0;
            }

            readFromBlock <short> (fptr); // fcnt
        }

        bcnt = readFromBlock <short> (fptr);
        while (bcnt-- > 0)
        {
            bytes[0] = readFromBlock <unsigned char> (fptr);
            bytes[1] = readFromBlock <unsigned char> (fptr);
            bytes[2] = readFromBlock <unsigned char> (fptr);

            *(xdst++) = 7 - ((bytes[2] >> 5) & 7);
            *(xdst++) = 7 - ((bytes[2] >> 2) & 7);
            *(xdst++) = 7 - ((bytes[1] >> 7) | ((bytes[2] << 1) & 6));
            *(xdst++) = 7 - ((bytes[1] >> 4) & 7);
            *(xdst++) = 7 - ((bytes[1] >> 1) & 7);
            *(xdst++) = 7 - ((bytes[0] >> 6) | ((bytes[1] << 2) & 4));
            *(xdst++) = 7 - ((bytes[0] >> 3) & 7);
            *(xdst++) = 7 - (bytes[0] & 7);

            x += 8;
            if (x >= 160)
            {
                x = 0;
                dst -= dstrowlen;
                xdst = dst;
            }
        }
    }
}



void AGSFile::loadFromAGSFile (const char* filename)
{
    ucl::win::WindowsHandle <HANDLE> hFile;
    ucl::stl::ArrayPtr <apvfileheader_t> header;
    ucl::stl::ArrayPtr <apvfileheader_t::fileinfo_t> subfileinfo;

    DWORD nobr;


    hFile = CreateFile (filename, GENERIC_READ, FILE_SHARE_READ,
        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (!hFile.isValid ())
        throw std::runtime_error ("Unable to open file!");

    header.alloc (1);

    ReadFile (hFile, header.get (), sizeof (apvfileheader_t), &nobr, NULL);

    if ((nobr != sizeof (apvfileheader_t))
      | ((header->version != 100) && (header->version != 101))
      | (std::memcmp (header->signature, "APVFILE1", 8) != 0))
        throw std::runtime_error ("File is not a valid APV file!");

    if (std::memcmp (header->filetype, ".AGS", 4) != 0)
        throw std::runtime_error ("File is not a AGS file!");

    if (header->filetypeversion != 101)
        throw std::runtime_error ("Unsupported AGS filetype version!");

    if (header->subfilecnt != 3)
        throw std::runtime_error ("Invalid number of subfiles!");

    subfileinfo.alloc (3);
    SetFilePointer (hFile, header->headerlen, NULL, FILE_BEGIN);
    ReadFile (hFile, subfileinfo.get (), sizeof (apvfileheader_t::fileinfo_t) * 3,
        &nobr, NULL);

    for (unsigned n = 0; n < 3; ++n)
    {
        files[n].alloc (subfileinfo[n].filesize);
        filesize[n] = subfileinfo[n].filesize;
        SetFilePointer (hFile, subfileinfo[n].fileoffs, NULL, FILE_BEGIN);
        ReadFile (hFile, files[n].get (), filesize[n], &nobr, NULL); 
    }

    _filename.assign (reinterpret_cast <char*> (files[0].get ())); // static_cast not allowed?!?
    for (char* p = &_filename[0], * l = p + _filename.size (); p < l; ++p)
        *p = static_cast <char> (charmap_pv2ansi[static_cast <unsigned char> (*p)]);
}

void AGSFile::saveToAGSFile (const char* filename)
{
    ucl::win::WindowsHandle <HANDLE> hFile;
    ucl::stl::ArrayPtr <char> file;
    apvfileheader_t* header;
    apvfileheader_t::fileinfo_t* subfileinfo;
    unsigned offs;
    DWORD nobw;


    if (filesize[0] == 0)
        return;

    hFile = CreateFile (filename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
        FILE_ATTRIBUTE_NORMAL, NULL);
    if (!hFile.isValid ())
        throw std::runtime_error ("Unable to write file!");

    file.alloc (sizeof (apvfileheader_t)
              + sizeof (apvfileheader_t::fileinfo_t) * 3);
    header = reinterpret_cast <apvfileheader_t*> (file.get ());
    subfileinfo = reinterpret_cast <apvfileheader_t::fileinfo_t*> (file.get ()
        + sizeof (apvfileheader_t));

    memcpy (header->signature, "APVFILE1", 8);
    header->version = 101;
    header->headerlen = sizeof (apvfileheader_t);
    memcpy (header->filetype, ".AGS", 4);
    header->filetypeversion = 101;
    header->subfilecnt = 3;
    header->flags = 0x00000000;
    header->mode = apvfileheader_t::moAddIn;

    offs = file.size ();

    for (unsigned n = 0; n < 3; ++n)
    {
        subfileinfo[n].fileoffs = offs;
        subfileinfo[n].filesize = filesize[n];
        offs += filesize[n];
    }

    WriteFile (hFile, file.get (), file.size (), &nobw, NULL);

    for (unsigned n = 0; n < 3; ++n)
        WriteFile (hFile, files[n].get (), filesize[n], &nobw, NULL);
}


} // namespace apv


