
/*
 *  agsfile.hpp
 *
 *  A class to convert bitmaps to PocketPicture .AGS files and vice versa
 *  Written by Moritz Beutel
 *  (c) 2007 by AUDACIA Software
 */

#ifndef _AGSFILE_HPP
#define _AGSFILE_HPP

#include "apvfile.hpp"
#include <ucl/stl/arrayptr.hpp>
#include <windows.h>
#include <string>

namespace apv
{

/*
 * .AGS file format:
 *
 * File 0:
 * =======
 * <filename$>
 * <version> <xsize> <ysize>
 * 8 * <rgbval>
 * <filecnt>
 * <3bytescnt>
 * 3bytescnt * (<byte0> <byte1> <byte2>)
 *
 * File 1..n:
 * ==========
 * <filename$>
 * <3bytescnt>
 * 3bytescnt * (<byte0> <byte1> <byte2>)
 *
 */


class AGSFile
{
    ucl::stl::ArrayPtr <unsigned char> files[3];
    unsigned filesize[3];
    std::string _filename;

public:
    AGSFile (void);
    AGSFile (const AGSFile& rhs);
    ~AGSFile (void);

    void pasteFromBitmap (const BITMAPINFOHEADER* bmih, const BYTE* bdata,
                          const char* filename);
    void getAsBitmap (ucl::stl::ArrayPtr <BITMAPINFOHEADER>& bmih,
                      ucl::stl::ArrayPtr <BYTE>& bdata);

    std::string getFileName (void)
    { return _filename; }

    void loadFromAGSFile (const char* filename);
    void saveToAGSFile (const char* filename);
};


} // namespace apv


#endif // _AGSFILE_HPP
