
/*
 *  apvfile.hpp
 *
 *  A file header definition for a CASIO PV container file format 
 *  Written by Moritz Beutel
 *  (c) 2007 by AUDACIA Software
 */


#ifndef _APVFILE_HPP
#define _APVFILE_HPP

namespace apv
{


/*
 * .APV file format:
 *
 * <apvfileheader>
 * header.subfilecnt * <fileinfo>
 *
 * fileinfo[n].fileoffs: <file n>
 *
 */

struct apvfileheader_t // v1.01
{
    struct fileinfo_t
    {
        unsigned int fileoffs;
        unsigned int filesize;
    };

    enum filetype_t
    {
        ftText          = 0x00,
        ftBinary        = 0x01,
    };

    static const filetype_t filetypes[];

    enum mode_t
    {
        moNone          = 0x00,
        moTel           = 0x01,
        moTelCompany    = 0x02,
        moMemo          = 0x03,
        moSchedule      = 0x04,
        moSpreadsheet   = 0x05,
        moMail          = 0x06,
        moExpense       = 0x07,
        moClock         = 0x08,
        moDualWindow    = 0x09,
        moAddIn         = 0x0A,
        moQuickMemo     = 0x0B,

        moMaxVal        = 0xFFFFFFFF,
    };


    unsigned char   signature[8];    // "APVFILE1"
    unsigned int    version;         // 100 | 101

    unsigned int    headerlen;

    char            filetype[4];     // typically uppercase file extension
    unsigned int    filetypeversion;

    unsigned int    subfilecnt;

        // version >= 101
    unsigned int    flags;           // default: 0
    mode_t          mode;            // default: 0x0A
};

const apvfileheader_t::filetype_t apvfileheader_t::filetypes[] = {
    ftBinary, ftText,   ftText,   ftText,   ftText,   ftBinary,
    ftBinary, ftText,   ftText,   ftText,   ftBinary, ftBinary,
};




} // namespace apv

#endif // _APVFILE_HPP
