
#include "fract.hpp"
#include <cmath>

#if (defined(__BCPLUSPLUS__) || defined(_MSCVER))
 #include <windows.h>
#endif

#ifdef _WINDOWS_
 #define _MULDIV(a,b,c) MulDiv ((a), (b), (c))
#else
 #define _MULDIV(a,b,c) static_cast <int> ((static_cast <long long> (a) * (b)) / (c))
#endif

namespace ucl
{

namespace math
{

int Fraction::_ggt (int a, int b)
{
    bool bl;
    int m, l, d;

    if (a == 0)
        return b;
    bl = std::abs (a) < std::abs (b);
    m = bl ? a : b;
    l = bl ? b : a;

    while (true)
    {
        d = l % m;
        if (d == 0)
            break;
        l = m;
        m = d;
    }

    return m;
}
int Fraction::_kgv (int a, int b)
{
    return _MULDIV (a, b, _ggt (a, b));
}

void Fraction::_opt (void)
{
    int div = _ggt (numerator, denominator);
    numerator /= div;
    denominator /= div;
    if (denominator < 0)
    {
        numerator = -numerator;
        denominator = -denominator;
    }
}

std::string Fraction::asString (void) const
{
    std::string rv;
    rv.resize (33);
    if (denominator != 1)
        std::sprintf (&rv[0], "%d/%d", numerator, denominator);
    else
        std::sprintf (&rv[0], "%d", numerator);
    return rv;
}

Fraction& Fraction::operator += (const Fraction& rhs)
{
    int k = _kgv (denominator, rhs.denominator);
    numerator = _MULDIV (numerator, k, denominator)
              + _MULDIV (rhs.numerator, k, rhs.denominator);
    denominator = k;
    _opt ();
    return *this;
}
Fraction& Fraction::operator -= (const Fraction& rhs)
{
    int k = _kgv (denominator, rhs.denominator);
    numerator = _MULDIV (numerator, k, denominator)
              - _MULDIV (rhs.numerator, k, rhs.denominator);
    denominator = k;
    _opt ();
    return *this;
}
Fraction& Fraction::operator *= (const Fraction& rhs)
{
    numerator *= rhs.numerator;
    denominator *= rhs.denominator;
    _opt ();
    return *this;
}
Fraction& Fraction::operator /= (const Fraction& rhs)
{
    numerator *= rhs.denominator;
    denominator *= rhs.numerator;
    _opt ();
    return *this;
}


} // namespace math

} // namespace ucl

