
#ifndef _UCL_FRACT_HPP
#define _UCL_FRACT_HPP

#include <string>
#include <cstdio>


namespace ucl
{

namespace math
{


class Fraction
{
private:
    int numerator;
    int denominator;

    static int _ggt (int a, int b);
    static int _kgv (int a, int b);

    void _opt (void);

public:
    Fraction (void)
     : numerator (0), denominator (1)
    {}
    explicit Fraction (int n, int d = 1)
     : numerator (n), denominator (d)
    { _opt (); }
    ~Fraction (void) {}

    int getNumerator (void) const { return numerator; }
    int getDenominator (void) const { return denominator; }

    double asDouble (void) const
    { return (static_cast <double> (numerator) / denominator); }
    std::string asString (void) const;

    Fraction& operator += (const Fraction& rhs);
    Fraction& operator -= (const Fraction& rhs);
    Fraction& operator *= (const Fraction& rhs);
    Fraction& operator /= (const Fraction& rhs);
};

Fraction& operator + (const Fraction& lhs, const Fraction& rhs)
{ return Fraction (lhs) += rhs; }
Fraction& operator - (const Fraction& lhs, const Fraction& rhs)
{ return Fraction (lhs) -= rhs; }
Fraction& operator * (const Fraction& lhs, const Fraction& rhs)
{ return Fraction (lhs) *= rhs; }
Fraction& operator / (const Fraction& lhs, const Fraction& rhs)
{ return Fraction (lhs) /= rhs; }

bool operator == (const Fraction& lhs, const Fraction& rhs)
{
    return ((lhs.getNumerator ()   == rhs.getNumerator ())
         && (lhs.getDenominator () == rhs.getDenominator ()));
}
bool operator != (const Fraction& lhs, const Fraction& rhs)
{
    return ((lhs.getNumerator ()   != rhs.getNumerator ())
         || (lhs.getDenominator () != rhs.getDenominator ()));
}
bool operator < (const Fraction& lhs, const Fraction& rhs)
{ return (rhs - lhs).getNumerator () < 0; }
bool operator > (const Fraction& lhs, const Fraction& rhs)
{ return (rhs - lhs).getNumerator () > 0; }
bool operator <= (const Fraction& lhs, const Fraction& rhs)
{ return (rhs - lhs).getNumerator () <= 0; }
bool operator >= (const Fraction& lhs, const Fraction& rhs)
{ return (rhs - lhs).getNumerator () >= 0; }


} // namespace math

} // namespace ucl

#endif // _UCL_FRACT_HPP
