//---------------------------------------------------------------------------

#include <vcl.h>
#include <windows.h>
#include <stdexcept>
#pragma hdrstop


#include "main_unit.h"
#include "grayscalefactors_unit.h"
#include "colors_unit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFrmMain *FrmMain;


//---------------------------------------------------------------------------
__fastcall TFrmMain::TFrmMain(TComponent* Owner)
    : TForm(Owner), gs_loaded (false)
{
    ctbl = agraph::colortable_t::createColorTable (8);
    ctbl->setToDefault ();

        // make EdtContrast and EdtBrightness only accept numbers
    SetWindowLong (EdtContrast->Handle, GWL_STYLE,
        GetWindowLong (EdtContrast->Handle, GWL_STYLE) | ES_NUMBER);
    SetWindowLong (EdtBrightness->Handle, GWL_STYLE,
        GetWindowLong (EdtBrightness->Handle, GWL_STYLE) | ES_NUMBER);
}
//---------------------------------------------------------------------------
void __fastcall TFrmMain::BtnCloseClick(TObject *Sender)
{
    Close ();    
}

//---------------------------------------------------------------------------
void __fastcall TFrmMain::BtnFileOpenClick(TObject *Sender)
{
    AnsiString ext;


    if (!OpdFile->Execute ())
        return;

    ext = ExtractFileExt (OpdFile->FileName).UpperCase ();

    if ((ext == ".BMP") || (ext == ".DIB"))
    {
        gcbm.loadFromBitmap (OpdFile->FileName.c_str (),
            agraph::GSBitmap::checkfile_relation <1, 1>);
        gcbm.performResize (160, 146, agraph::GSBitmap::reBilinear);

        EdtPVFileName->Text = ChangeFileExt (ExtractFileName (OpdFile->FileName), "");
    }
    else if (ext == ".AGS")
    {
        ucl::stl::ArrayPtr <BITMAPINFOHEADER> bmih;
        ucl::stl::ArrayPtr <BYTE> bdata;


        agsfile.loadFromAGSFile (OpdFile->FileName.c_str ());
        agsfile.getAsBitmap (bmih, bdata);
        gcbm.pasteFromBitmap (bmih.get (), bdata.get ());

        EdtPVFileName->Text = ChangeFileExt (agsfile.getFileName ().c_str (), "");
    }
    else
        throw Exception ("Unknown file type!");

    EdtFileName->Text = OpdFile->FileName;
    
    gs_loaded = true;
    gcbm.drawToDevice (ImgOriginal->Canvas->Handle);
    UpdateGrayscaledFile (Sender);
    ImgOriginal->Refresh ();

    EdtPVFileNameChange (Sender);
    BtnSaveBMP->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TFrmMain::BtnSaveBMPClick(TObject *Sender)
{
    AnsiString ext;

    if (!SvdBMPFile->Execute ())
        return;

    ext = ExtractFileExt (SvdBMPFile->FileName).LowerCase ();
    if ((ext != ".bmp") && (ext != ".dib"))
        SvdBMPFile->FileName = SvdBMPFile->FileName + ".bmp";

    gsbm.saveToBitmap (SvdBMPFile->FileName.c_str ());
}
//---------------------------------------------------------------------------
void __fastcall TFrmMain::BtnSaveAGSClick(TObject *Sender)
{
    AnsiString ext;

    if (!SvdAGSFile->Execute ())
        return;

    ext = ExtractFileExt (SvdAGSFile->FileName).LowerCase ();
    if (ext != ".ags")
        SvdAGSFile->FileName = SvdAGSFile->FileName + ".ags";

    agsfile.pasteFromBitmap (&(gsbm.getBitmapInfo ()->bmiHeader),
        gsbm.getBitmap (), EdtPVFileName->Text.c_str ());
    agsfile.saveToAGSFile (SvdAGSFile->FileName.c_str ());
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::Button1Click(TObject *Sender)
{
    MessageBox (Handle, "PictureConverter for PV v1.05\n"
                        "by Moritz Beutel\n"
                        "(c) 2007 by AUDACIA Software\n"
                        "www.audacia-software.de\n\n"
                        "Created with Borland C++Builder 2006.\n\n"
                        "Use this program to export grayscaled pictures for PocketPicture.",
                "About PictureConverter for PV", MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------


void __fastcall TFrmMain::UpdateGrayscaledFile(TObject *Sender)
{
    if (!gs_loaded)
        return;

    gsbm = gcbm;
    gsbm.performGrayscaleConversion (ctbl.get (), cgsf,
        agraph::GSBitmap::contrast_t (EdtContrast->Text.ToInt (),
            EdtBrightness->Text.ToInt ()),
        ChkFloydSteinberg->Checked
            ? agraph::GSBitmap::diFloydSteinberg
            : agraph::GSBitmap::diNone);

    gsbm.drawToDevice (ImgGrayscale->Canvas->Handle);
    ImgGrayscale->Refresh ();
}
//---------------------------------------------------------------------------



void __fastcall TFrmMain::EdtPVFileNameChange(TObject *Sender)
{
    if (EdtPVFileName->Text.IsEmpty ())
        BtnSaveAGS->Enabled = false;
    else
        BtnSaveAGS->Enabled = (EdtPVFileName->Text.Length () < 40)
                            ? gs_loaded
                            : false;
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::BtnResetClick(TObject *Sender)
{
    bool gsl = gs_loaded;
    gs_loaded = false;

    EdtContrast->Text = "100";
    EdtBrightness->Text = "0";
    cgsf.setToDefault ();
    ctbl->setToDefault ();
    ChkFloydSteinberg->Checked = true;

    gs_loaded = gsl;
    UpdateGrayscaledFile (Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::BtnAdjustFactorsClick(TObject *Sender)
{
    if (FrmGrayscaleFactors->invoke (cgsf))
        UpdateGrayscaledFile (Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::BtnAdjustColorsClick(TObject *Sender)
{
    if (FrmColors->invoke (ctbl.get ()))
        UpdateGrayscaledFile (Sender);
}
//---------------------------------------------------------------------------

