

#ifdef _DOXYGEN

namespace ucl
{

/**
 *  @brief   Borland C++ specific classes and functions
 *  @author  Moritz Beutel
 *
 *  This namespace contains classes and functions specific to
 *  Borland C++ (BCC) and C++Builder, e.g. the MulticastClosure
 *  classes that allow to use multiple event handlers with
 *  the __closure language extension.
 *
 *  Files:
 *  - <b>ucl/bcc/multicastsupport.hpp</b> - Lightweight header file
 *    that contains the basic support classes for MulticastClosures.
 *    Should be included by header files that define a
 *    MulticastContainer object
 *  - <b>ucl/bcc/multicast.hpp</b> - The definition of MulticastClosureN
 *    and the supporting functions. Required for the use of multicasts
 *  - <b>ucl/bcc/multicastspec.hpp</b> - defines a few specializations
 *    of the asMulticast() function for very common closure types, e.g.:
 *      -# void __fastcall (__closure* TNotifyEvent) (System::TObject*)
 */
namespace bcc {}

/**
 *  @brief   STL supporting classes
 *  @author  Moritz Beutel
 *
 *  This namespace contains several classes that extend the
 *  variety of STL containers, algorithms, iterators or
 *  definitions.
 *
 *  Files:
 *  - <b>ucl/stl/meta.hpp</b> - A few template metaclass definitions
 *  - <b>ucl/stl/arrayptr.hpp</b> - A smart pointer very
 *  similar to std::auto_ptr, but with support for heap arrays
 *  - <b>ucl/stl/iterwrap.hpp</b> - A bidirectional and a random
 *    iterator wrapper with support for plain pointer wrapping and
 *    iterator debugging
 *  - <b>ucl/stl/arraywrap.hpp</b> - An array wrapper container
 */
namespace stl {}

}

/** @mainpage ucl Documentation
 *  @author   Moritz Beutel
 *  @section intro_sec Introduction
 *
 *  <b>ucl</b> is one of many libraries with the intention to
 *  simplify C++ programming. It provides classes and functions
 *  that supplement the STL and CodeGear's VCL (Visual Component
 *  Library).
 *  
 *  Although this library may intersect with some popular libraries
 *  such as Boost, it does not intend to compete with them. The
 *  simple reason for redefining a few of the things that are
 *  present in Boost is that I wanted this library to not depend
 *  on another library itself.
 *
 *
 *
 *  @section install_sec Installation
 *
 *  The provided .zip file includes the following directories:
 *  - <b>/include</b> - the header files. Extract this to the
 *    include path of your compiler, or extract it somewhere else
 *    and add the path to the compiler's search path
 *  - <b>/source</b> - the source files. Extract this somewhere and
 *    add the sources which you require to your projects.
 *    Alternatively, you can compile them as a static library.
 *  - <b>/doc</b> - the documentation files
 *  - <b>/samples</b> - a few sample applications that demonstrate
 *    the usage of <b>ucl</b>.
 *
 *
 *  @section license_sec License
 *
 *  By using this software, you agree with the following conditions:
 *  - All supplied source, header, documentation and binary files are
 *    the intellectual property of AUDACIA Software and their authors,
 *    as long as not marked otherwise.
 *  - This software is provided "as-is" without any warranty.
 *    Neither AUDACIA Software nor the author bear any kind of
 *    responsibility for damages, losses or other problems caused by
 *    this software.
 *  - This library may be used for any purpose, including commercial
 *    applications, as long as the following conditions are fulfilled:
 *      -# You must not claim that you are the author of this software.
 *      -# Modified versions of this library must be clearly marked
 *         as such. You must not claim that you have written the
 *         original version. Furthermore, you must give a reference to
 *         the original version.
 *  - If you use this software in your project, a reference to
 *    AUDACIA Software is not required but appreciated.
 *
 *
 *  @section contact_sec Contact
 *
 *  To download the most recent version of <b>ucl</b>, visit
 *  http://www.audacia-software.de/ . If you have questions or
 *  suggestions, <a href="mailto:mb@audacia-software.de">contact me</a>.
 *
 */



#endif