
#include <algorithm>

#include <ucl/bcc/multicast.hpp>
#pragma hdrstop


bool operator == (const TMethod& lhs, const TMethod& rhs)
{ return ((lhs.Code == rhs.Code) && (lhs.Data == rhs.Data)); }
bool operator != (const TMethod& lhs, const TMethod& rhs)
{ return ((lhs.Code != rhs.Code) || (lhs.Data != rhs.Data)); }

namespace ucl
{

namespace bcc
{


MulticastContainer::~MulticastContainer (void)
{
    if (first)
        first->destroy ();
}

namespace detail
{

const unsigned MulticastClosureBase::MCEBSignature = 0xFEDCBA98;

MulticastClosureBase* MulticastClosureBase::registerMulticastClosure (
    MulticastContainer& mcc, TMethod* event, const TMethod* mce, MulticastClosureBase* mceb)
{
    if (!(mcc.first))
        mcc.first = mcc.last = mceb;
    else
    {
        mcc.last->next = mceb;
        mcc.last = mceb;
    }

    if (event->Code)
        mceb->list.push_back (*event);

    mceb->myEvent = event;   // for setBreakable
    event->Code = mce->Code; // MulticastClosureN::operator ()
    event->Data = mce->Data; // MulticastClosureN object

    return mceb;
}


MulticastClosureBase::MulticastClosureBase (MulticastDestructor md, TMethod* event)
 : next (NULL), signature (MCEBSignature), mcd (md), myEvent (event) { }
MulticastClosureBase::~MulticastClosureBase (void)
{
    if (next)
        next->destroy ();
}


MulticastClosureBase::list_t::iterator MulticastClosureBase::_insert
    (list_t::iterator i, const void* e)
{
    const TMethod* meth = reinterpret_cast <const TMethod*> (e);
    return (list.insert (i, *meth));
}
MulticastClosureBase::list_t::iterator MulticastClosureBase::_erase (list_t::iterator i)
{ return (list.erase (i)); }
void MulticastClosureBase::_push_back (const void* e)
{
    const TMethod* meth = reinterpret_cast <const TMethod*> (e);
    list.push_back (*meth);
}
void MulticastClosureBase::_push_front (const void* e)
{
    const TMethod* meth = reinterpret_cast <const TMethod*> (e);
    list.push_front (*meth);
}
void MulticastClosureBase::_remove (const void* e)
{ list.remove (*reinterpret_cast <const TMethod*> (e)); }
bool MulticastClosureBase::_contains (const void* e)
{
    const TMethod* meth = reinterpret_cast <const TMethod*> (e);
    list_t::iterator end = list.end ();
    return (std::find (list.begin (), end, *meth) != end);
}
void MulticastClosureBase::_swap (MulticastClosureBase& rhs)
{ list.swap (rhs.list); }

void MulticastClosureBase::clear (void)
{ list.clear (); }
void MulticastClosureBase::pop_back (void)
{ list.pop_back (); }
void MulticastClosureBase::pop_front (void)
{ list.pop_front (); }



} // namespace bcc
} // namespace detail
} // namespace ucl


