
#ifndef _UCL_WINUTIL_HPP
#define _UCL_WINUTIL_HPP

#include <windows.h>


namespace ucl
{

namespace win
{


typedef BOOL WINAPI (*closehandlefunc_t) (void*);
template <typename T = HANDLE,
          closehandlefunc_t closehandle = CloseHandle,
          T invalid_value = INVALID_HANDLE_VALUE>
    class WindowsHandle
{
private:
    T handle_;

public:
    WindowsHandle (void)
     : handle_ (invalid_value)
    {}
    WindowsHandle (const WindowsHandle& rhs)
     : handle_ (rhs.handle_)
    {}
    WindowsHandle (T WindowsHandle)
     : handle_ (WindowsHandle)
    {}
    ~WindowsHandle (void)
    {
        if (handle_ != invalid_value)
        {
            closehandle ((void*) handle_);
            handle_ = invalid_value;
        }
    }

    void close (void)
    {
        if (handle_ != invalid_value)
        {
            closehandle ((void*) handle_);
            handle_ = invalid_value;
        }
    }

    const WindowsHandle& operator = (const WindowsHandle& rhs)
    {
        close ();
        handle_ = rhs.handle_;
    }

    const WindowsHandle& operator = (T WindowsHandle)
    {
        close ();
        handle_ = WindowsHandle;
    }

    operator T (void) const { return handle_; }

    bool isValid (void) const { return (handle_ != invalid_value); }
};


template <typename T, closehandlefunc_t ch, T iv>
    bool operator == (const WindowsHandle <T, ch, iv>& lhs,
                      const WindowsHandle <T, ch, iv>& rhs)
{ return lhs.operator T () == rhs.operator T (); }

template <typename T, closehandlefunc_t ch, T iv>
    bool operator != (const WindowsHandle <T, ch, iv>& lhs,
                      const WindowsHandle <T, ch, iv>& rhs)
{ return lhs.operator T () != rhs.operator T (); }


} // namespace win

} // namespace ucl

#endif // _UCL_WINUTIL_HPP

